/*******************************************************************************
* File Name: XaxCtr.c  
* Version 3.0
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "XaxCtr.h"

uint8 XaxCtr_initVar = 0u;


/*******************************************************************************
* Function Name: XaxCtr_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void XaxCtr_Init(void) 
{
        #if (!XaxCtr_UsingFixedFunction && !XaxCtr_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!XaxCtr_UsingFixedFunction && !XaxCtr_ControlRegRemoved) */
        
        #if(!XaxCtr_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 XaxCtr_interruptState;
        #endif /* (!XaxCtr_UsingFixedFunction) */
        
        #if (XaxCtr_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            XaxCtr_CONTROL &= XaxCtr_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                XaxCtr_CONTROL2 &= ((uint8)(~XaxCtr_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                XaxCtr_CONTROL3 &= ((uint8)(~XaxCtr_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (XaxCtr_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                XaxCtr_CONTROL |= XaxCtr_ONESHOT;
            #endif /* (XaxCtr_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            XaxCtr_CONTROL2 |= XaxCtr_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            XaxCtr_RT1 &= ((uint8)(~XaxCtr_RT1_MASK));
            XaxCtr_RT1 |= XaxCtr_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            XaxCtr_RT1 &= ((uint8)(~XaxCtr_SYNCDSI_MASK));
            XaxCtr_RT1 |= XaxCtr_SYNCDSI_EN;

        #else
            #if(!XaxCtr_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = XaxCtr_CONTROL & ((uint8)(~XaxCtr_CTRL_CMPMODE_MASK));
            XaxCtr_CONTROL = ctrl | XaxCtr_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = XaxCtr_CONTROL & ((uint8)(~XaxCtr_CTRL_CAPMODE_MASK));
            
            #if( 0 != XaxCtr_CAPTURE_MODE_CONF)
                XaxCtr_CONTROL = ctrl | XaxCtr_DEFAULT_CAPTURE_MODE;
            #else
                XaxCtr_CONTROL = ctrl;
            #endif /* 0 != XaxCtr_CAPTURE_MODE */ 
            
            #endif /* (!XaxCtr_ControlRegRemoved) */
        #endif /* (XaxCtr_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!XaxCtr_UsingFixedFunction)
            XaxCtr_ClearFIFO();
        #endif /* (!XaxCtr_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        XaxCtr_WritePeriod(XaxCtr_INIT_PERIOD_VALUE);
        #if (!(XaxCtr_UsingFixedFunction && (CY_PSOC5A)))
            XaxCtr_WriteCounter(XaxCtr_INIT_COUNTER_VALUE);
        #endif /* (!(XaxCtr_UsingFixedFunction && (CY_PSOC5A))) */
        XaxCtr_SetInterruptMode(XaxCtr_INIT_INTERRUPTS_MASK);
        
        #if (!XaxCtr_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)XaxCtr_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            XaxCtr_WriteCompare(XaxCtr_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            XaxCtr_interruptState = CyEnterCriticalSection();
            
            XaxCtr_STATUS_AUX_CTRL |= XaxCtr_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(XaxCtr_interruptState);
            
        #endif /* (!XaxCtr_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: XaxCtr_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void XaxCtr_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (XaxCtr_UsingFixedFunction)
        XaxCtr_GLOBAL_ENABLE |= XaxCtr_BLOCK_EN_MASK;
        XaxCtr_GLOBAL_STBY_ENABLE |= XaxCtr_BLOCK_STBY_EN_MASK;
    #endif /* (XaxCtr_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!XaxCtr_ControlRegRemoved || XaxCtr_UsingFixedFunction)
        XaxCtr_CONTROL |= XaxCtr_CTRL_ENABLE;                
    #endif /* (!XaxCtr_ControlRegRemoved || XaxCtr_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: XaxCtr_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  XaxCtr_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void XaxCtr_Start(void) 
{
    if(XaxCtr_initVar == 0u)
    {
        XaxCtr_Init();
        
        XaxCtr_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    XaxCtr_Enable();        
}


/*******************************************************************************
* Function Name: XaxCtr_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void XaxCtr_Stop(void) 
{
    /* Disable Counter */
    #if(!XaxCtr_ControlRegRemoved || XaxCtr_UsingFixedFunction)
        XaxCtr_CONTROL &= ((uint8)(~XaxCtr_CTRL_ENABLE));        
    #endif /* (!XaxCtr_ControlRegRemoved || XaxCtr_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (XaxCtr_UsingFixedFunction)
        XaxCtr_GLOBAL_ENABLE &= ((uint8)(~XaxCtr_BLOCK_EN_MASK));
        XaxCtr_GLOBAL_STBY_ENABLE &= ((uint8)(~XaxCtr_BLOCK_STBY_EN_MASK));
    #endif /* (XaxCtr_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: XaxCtr_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void XaxCtr_SetInterruptMode(uint8 interruptsMask) 
{
    XaxCtr_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: XaxCtr_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   XaxCtr_ReadStatusRegister(void) 
{
    return XaxCtr_STATUS;
}


#if(!XaxCtr_ControlRegRemoved)
/*******************************************************************************
* Function Name: XaxCtr_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   XaxCtr_ReadControlRegister(void) 
{
    return XaxCtr_CONTROL;
}


/*******************************************************************************
* Function Name: XaxCtr_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    XaxCtr_WriteControlRegister(uint8 control) 
{
    XaxCtr_CONTROL = control;
}

#endif  /* (!XaxCtr_ControlRegRemoved) */


#if (!(XaxCtr_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: XaxCtr_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void XaxCtr_WriteCounter(uint32 counter) \
                                   
{
    #if(XaxCtr_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (XaxCtr_GLOBAL_ENABLE & XaxCtr_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        XaxCtr_GLOBAL_ENABLE |= XaxCtr_BLOCK_EN_MASK;
        CY_SET_REG16(XaxCtr_COUNTER_LSB_PTR, (uint16)counter);
        XaxCtr_GLOBAL_ENABLE &= ((uint8)(~XaxCtr_BLOCK_EN_MASK));
    #else
        CY_SET_REG24(XaxCtr_COUNTER_LSB_PTR, counter);
    #endif /* (XaxCtr_UsingFixedFunction) */
}
#endif /* (!(XaxCtr_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: XaxCtr_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) The present value of the counter.
*
*******************************************************************************/
uint32 XaxCtr_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(XaxCtr_UsingFixedFunction)
		(void)CY_GET_REG16(XaxCtr_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(XaxCtr_COUNTER_LSB_PTR_8BIT);
	#endif/* (XaxCtr_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(XaxCtr_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(XaxCtr_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG24(XaxCtr_STATICCOUNT_LSB_PTR));
    #endif /* (XaxCtr_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: XaxCtr_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint32) Present Capture value.
*
*******************************************************************************/
uint32 XaxCtr_ReadCapture(void) 
{
    #if(XaxCtr_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(XaxCtr_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG24(XaxCtr_STATICCOUNT_LSB_PTR));
    #endif /* (XaxCtr_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: XaxCtr_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint32) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void XaxCtr_WritePeriod(uint32 period) 
{
    #if(XaxCtr_UsingFixedFunction)
        CY_SET_REG16(XaxCtr_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG24(XaxCtr_PERIOD_LSB_PTR, period);
    #endif /* (XaxCtr_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: XaxCtr_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) Present period value.
*
*******************************************************************************/
uint32 XaxCtr_ReadPeriod(void) 
{
    #if(XaxCtr_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(XaxCtr_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG24(XaxCtr_PERIOD_LSB_PTR));
    #endif /* (XaxCtr_UsingFixedFunction) */
}


#if (!XaxCtr_UsingFixedFunction)
/*******************************************************************************
* Function Name: XaxCtr_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void XaxCtr_WriteCompare(uint32 compare) \
                                   
{
    #if(XaxCtr_UsingFixedFunction)
        CY_SET_REG16(XaxCtr_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG24(XaxCtr_COMPARE_LSB_PTR, compare);
    #endif /* (XaxCtr_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: XaxCtr_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint32) Present compare value.
*
*******************************************************************************/
uint32 XaxCtr_ReadCompare(void) 
{
    return (CY_GET_REG24(XaxCtr_COMPARE_LSB_PTR));
}


#if (XaxCtr_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: XaxCtr_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void XaxCtr_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    XaxCtr_CONTROL &= ((uint8)(~XaxCtr_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    XaxCtr_CONTROL |= compareMode;
}
#endif  /* (XaxCtr_COMPARE_MODE_SOFTWARE) */


#if (XaxCtr_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: XaxCtr_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void XaxCtr_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    XaxCtr_CONTROL &= ((uint8)(~XaxCtr_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    XaxCtr_CONTROL |= ((uint8)((uint8)captureMode << XaxCtr_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (XaxCtr_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: XaxCtr_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void XaxCtr_ClearFIFO(void) 
{

    while(0u != (XaxCtr_ReadStatusRegister() & XaxCtr_STATUS_FIFONEMP))
    {
        (void)XaxCtr_ReadCapture();
    }

}
#endif  /* (!XaxCtr_UsingFixedFunction) */


/* [] END OF FILE */

