/*******************************************************************************
* File Name: XaxCtr_PM.c  
* Version 3.0
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "XaxCtr.h"

static XaxCtr_backupStruct XaxCtr_backup;


/*******************************************************************************
* Function Name: XaxCtr_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  XaxCtr_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void XaxCtr_SaveConfig(void) 
{
    #if (!XaxCtr_UsingFixedFunction)

        XaxCtr_backup.CounterUdb = XaxCtr_ReadCounter();

        #if(!XaxCtr_ControlRegRemoved)
            XaxCtr_backup.CounterControlRegister = XaxCtr_ReadControlRegister();
        #endif /* (!XaxCtr_ControlRegRemoved) */

    #endif /* (!XaxCtr_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: XaxCtr_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  XaxCtr_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void XaxCtr_RestoreConfig(void) 
{      
    #if (!XaxCtr_UsingFixedFunction)

       XaxCtr_WriteCounter(XaxCtr_backup.CounterUdb);

        #if(!XaxCtr_ControlRegRemoved)
            XaxCtr_WriteControlRegister(XaxCtr_backup.CounterControlRegister);
        #endif /* (!XaxCtr_ControlRegRemoved) */

    #endif /* (!XaxCtr_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: XaxCtr_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  XaxCtr_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void XaxCtr_Sleep(void) 
{
    #if(!XaxCtr_ControlRegRemoved)
        /* Save Counter's enable state */
        if(XaxCtr_CTRL_ENABLE == (XaxCtr_CONTROL & XaxCtr_CTRL_ENABLE))
        {
            /* Counter is enabled */
            XaxCtr_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            XaxCtr_backup.CounterEnableState = 0u;
        }
    #else
        XaxCtr_backup.CounterEnableState = 1u;
        if(XaxCtr_backup.CounterEnableState != 0u)
        {
            XaxCtr_backup.CounterEnableState = 0u;
        }
    #endif /* (!XaxCtr_ControlRegRemoved) */
    
    XaxCtr_Stop();
    XaxCtr_SaveConfig();
}


/*******************************************************************************
* Function Name: XaxCtr_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  XaxCtr_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void XaxCtr_Wakeup(void) 
{
    XaxCtr_RestoreConfig();
    #if(!XaxCtr_ControlRegRemoved)
        if(XaxCtr_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            XaxCtr_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!XaxCtr_ControlRegRemoved) */
    
}


/* [] END OF FILE */
